/*
 * Utility for sending CTRL-C events to programs with attached console only.
 * This simple utility is coded mainly for being able to send CTRL-C to Maxima,
 * expecting that Maxima has both console attached and catches SIGINT signals
 * under Wine-like system; though not sure on that, since not using ReactOS.
 *
 * Notice that I actually tested it under Wine with Tcl/Tk application (Minima),
 * having console application (SBCL-based Maxima) as its child process,
 * and CTRL-C was sent the self-way (see in the code below);
 * It worked correctly; with the (Tcl/Tk) parent process needed
 * to have CTRL-C caught by itself too (done by TWAPI for Tcl/Tk).
 *
 * This is free software, set into public domain.
 * THERE IS ABSOLUTELY NO (NEITHER IMPLIED) WARRANTY.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <windows.h>

int main(int argc, char **argv) {
  int pid;
  FILE *output = NULL;
  int status = 0;

  if ((argc==2) && (strcmp(argv[1],"--help")==0)) {
    output=stdout;
  }
  if (argc>3) {
    output=stderr;
    status = 1;
  }

  if (output!=NULL) {
    fprintf(output, "expecting either zero, one parameter (PID) or two parameters ([-]SIGNUM PID)\n");
    fprintf(output, "if zero parameters, this sends CTRL-C to processes of its own console,\n");
    fprintf(output, "otherwise it attaches to console of the specified PID process first.\n");
    fprintf(output, "the only signal delivered here is CTRL-C, that is SIGINT with signum == 2\n");
#ifdef SELF_ONLY
    fprintf(output, "this version does not do re-attaching (effectively ignoring given PID)\n");
    fprintf(output, "and thus only works correctly for child processes with inherited console.\n");
#endif
    return status;
  }

  if (argc==3) {
    if ((atoi(argv[1]) != -2) && (atoi(argv[1]) != 2)) {
      fprintf(stderr, "the SIGNUM, if specified, has to be equal to SIGINT, that is numerically 2\n");
      return 1;
    }
  }

// under wine (where I do testing of this) AttachConsole is not implemented,
// though child processes inherit console (if any, and it is e.g. for Tcl/Tk);
// thus sending CTRL-C to child processes does not need the re-attaching;
// and just avoiding it, we can do it the common way alike with kill;
#ifndef SELF_ONLY
  if (argc>1) {
    if (argc==2) {
      pid = atoi(argv[1]);
    } {
      pid = atoi(argv[2]);
    }
    FreeConsole(); // to make sure we do not have any console attached
    AttachConsole(pid); // to be able to send CTRL-C to the respected process
  }
#endif

  // if set FALSE below, this caller exits on the CTRL-C event,
  // if set TRUE below, this caller ignores the CTRL-C event, and exits then the standard way;
  // using the return-based exit here
  SetConsoleCtrlHandler(NULL, TRUE);
  GenerateConsoleCtrlEvent(CTRL_C_EVENT, 0); // sending CTRL-C event to processes of the attached console

  return 0;
}



